@@ -39,15 +39,17 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
-#include <libgnome/gnome-url.h>
-#include <libgnomeui/gnome-url.h>
+#include <libpanel-util/panel-error.h>
+#include <libpanel-util/panel-glib.h>
+#include <libpanel-util/panel-keyfile.h>
+#include <libpanel-util/panel-launch.h>
+#include <libpanel-util/panel-show.h>
 
 #include "menu.h"
 #include "panel-action-button.h"
 #include "panel-globals.h"
 #include "panel-icon-names.h"
 #include "panel-lockdown.h"
-#include "panel-mount-operation.h"
 #include "panel-recent.h"
 #include "panel-stock-icons.h"
 #include "panel-util.h"
@@ -60,8 +62,8 @@
 #define COMPUTER_NAME_KEY       "/apps/nautilus/desktop/computer_icon_name"
 #define MAX_ITEMS_OR_SUBMENU    5
 
-G_DEFINE_TYPE (PanelPlaceMenuItem, panel_place_menu_item, GTK_TYPE_IMAGE_MENU_ITEM);
-G_DEFINE_TYPE (PanelDesktopMenuItem, panel_desktop_menu_item, GTK_TYPE_IMAGE_MENU_ITEM);
+G_DEFINE_TYPE (PanelPlaceMenuItem, panel_place_menu_item, GTK_TYPE_IMAGE_MENU_ITEM)
+G_DEFINE_TYPE (PanelDesktopMenuItem, panel_desktop_menu_item, GTK_TYPE_IMAGE_MENU_ITEM)
 
 #define PANEL_PLACE_MENU_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_PLACE_MENU_ITEM, PanelPlaceMenuItemPrivate))
 #define PANEL_DESKTOP_MENU_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_DESKTOP_MENU_ITEM, PanelDesktopMenuItemPrivate))
@@ -97,67 +99,43 @@ struct _PanelDesktopMenuItemPrivate {
 };
 
 static void
-activate_uri (GtkWidget  *menuitem,
-	      const char *path)
+activate_uri_on_screen (const char *uri,
+			GdkScreen  *screen)
 {
-	GError    *error = NULL;
-	GFile     *file;
-	GdkScreen *screen;
-	char      *escaped;
-	char      *scheme;
-	char      *url;
+	panel_show_uri (screen, uri, gtk_get_current_event_time (), NULL);
+}
 
-	screen = menuitem_to_screen (menuitem);
-	
-	scheme = g_uri_get_scheme (path);
-	if (scheme) {
-		url = g_strdup (path);
-		g_free (scheme);
-	} else {
-		file = g_file_new_for_path (path);
-		url = g_file_get_uri (file);
-		g_object_unref (file);
-	}
-	
-	if (g_str_has_prefix (url, "x-nautilus-search:")) {
-		//FIXME: this is ugly...
-		char *command;
+static void
+activate_uri (GtkWidget  *menuitem,
+	      const char *uri)
+{
+	activate_uri_on_screen (uri, menuitem_to_screen (menuitem));
+}
 
-		command = g_strdup_printf ("nautilus --no-desktop %s", url);
-		gdk_spawn_command_line_on_screen (screen, command, &error);
-	} else
-		gnome_url_show_on_screen (url, screen, &error);
+static void
+activate_path (GtkWidget  *menuitem,
+	       const char *path)
+{
+	char *uri;
 
-	if (error != NULL) {
-		if (error->code != GNOME_URL_ERROR_CANCELLED) {
-			char *primary;
-			escaped = g_markup_escape_text (url, -1);
-			primary = g_strdup_printf (_("Could not open location '%s'"),
-						   escaped);
-			g_free (escaped);
-			panel_error_dialog (NULL, screen,
-					    "cannot_show_url", TRUE,
-					    primary, error->message);
-			g_free (primary);
-		}
-		g_error_free (error);
-	}
-	g_free (url);
+	uri = g_filename_to_uri (path, NULL, NULL);
+	activate_uri_on_screen (uri, menuitem_to_screen (menuitem));
+	g_free (uri);
 }
 
 static void
 activate_home_uri (GtkWidget *menuitem,
 		   gpointer   data)
 {
-	activate_uri (menuitem, g_get_home_dir ());
+	activate_path (menuitem, g_get_home_dir ());
 }
 
 static void
 activate_desktop_uri (GtkWidget *menuitem,
 		      gpointer   data)
 {
-	activate_uri (menuitem,
-		      g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
+	activate_path (menuitem,
+		       g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
 }
  
 static void
@@ -171,6 +149,9 @@ panel_menu_items_append_from_desktop (GtkWidget *menu,
 	char      *path_freeme;
 	char      *full_path;
 	char      *uri;
+	char      *type;
+	gboolean   is_application;
+	char      *tryexec;
 	char      *icon;
 	char      *name;
 	char      *comment;
@@ -213,17 +194,52 @@ panel_menu_items_append_from_desktop (GtkWidget *menu,
 		return;
 	}
 
-	icon    = panel_util_key_file_get_locale_string (key_file, "Icon");
-	comment = panel_util_key_file_get_locale_string (key_file, "Comment");
+	/* For Application desktop files, respect TryExec */
+	type = panel_key_file_get_string (key_file, "Type");
+	if (!type) {
+		g_key_file_free (key_file);
+		if (path_freeme)
+			g_free (path_freeme);
+		return;
+	}
+	is_application = (strcmp (type, "Application") == 0);
+	g_free (type);
 
-	if (string_empty (force_name))
-		name = panel_util_key_file_get_locale_string (key_file, "Name");
+	if (is_application) {
+		tryexec = panel_key_file_get_string (key_file, "TryExec");
+		if (tryexec) {
+			char *prog;
+
+			prog = g_find_program_in_path (tryexec);
+			g_free (tryexec);
+
+			if (!prog) {
+				/* FIXME: we could add some file monitor magic,
+				 * so that the menu items appears when the
+				 * program appears, but that's really complex
+				 * for not a huge benefit */
+				g_key_file_free (key_file);
+				if (path_freeme)
+					g_free (path_freeme);
+				return;
+			}
+
+			g_free (prog);
+		}
+	}
+
+	/* Now, simply build the menu item */
+	icon    = panel_key_file_get_locale_string (key_file, "Icon");
+	comment = panel_key_file_get_locale_string (key_file, "Comment");
+
+	if (PANEL_GLIB_STR_EMPTY (force_name))
+		name = panel_key_file_get_locale_string (key_file, "Name");
 	else
 		name = g_strdup (force_name);
 
 	item = gtk_image_menu_item_new ();
 	setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
-				   icon, NULL, name);
+				   icon, NULL, NULL, name);
 
 	panel_util_set_tooltip_text (item, comment);
 
@@ -257,6 +273,7 @@ panel_menu_items_append_from_desktop (GtkWidget *menu,
 
 static void
 panel_menu_items_append_place_item (const char *icon_name,
+				    GIcon      *gicon,
 				    const char *title,
 				    const char *tooltip,
 				    GtkWidget  *menu,
@@ -269,8 +286,7 @@ panel_menu_items_append_place_item (const char *icon_name,
 	item = gtk_image_menu_item_new ();
 	setup_menu_item_with_icon (item,
 				   panel_menu_icon_get_size (),
-				   icon_name,
-				   NULL,
+				   icon_name, NULL, gicon,
 				   title);
 
 	panel_util_set_tooltip_text (item, tooltip);
@@ -301,7 +317,7 @@ panel_menu_items_create_action_item_full (PanelActionButtonType  action_type,
         setup_menu_item_with_icon (item,
 				   panel_menu_icon_get_size (),
 				   panel_action_get_icon_name (action_type),
-				   NULL,
+				   NULL, NULL,
 				   label ? label : panel_action_get_text (action_type));
 
 	panel_util_set_tooltip_text (item,
@@ -412,7 +428,7 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 
 		item = gtk_image_menu_item_new ();
 		setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
-					   PANEL_ICON_BOOKMARKS, NULL,
+					   PANEL_ICON_BOOKMARKS, NULL, NULL,
 					   _("Bookmarks"));
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -461,11 +477,13 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 		}
 
 		icon = panel_util_get_icon_for_uri (bookmark->full_uri);
+		/*FIXME: we should probably get a GIcon if possible, so that we
+		 * have customized icons for cd-rom, eg */
 		if (!icon)
 			icon = g_strdup (PANEL_ICON_FOLDER);
 
 		//FIXME: drag and drop will be broken for x-nautilus-search uris
-		panel_menu_items_append_place_item (icon,
+		panel_menu_items_append_place_item (icon, NULL,
 						    label,
 						    tooltip,
 						    add_menu,
@@ -531,28 +549,23 @@ panel_menu_item_append_drive (GtkWidget *menu,
 {
 	GtkWidget *item;
 	GIcon     *icon;
-	char      *icon_name;
 	char      *title;
 	char      *tooltip;
 
 	icon = g_drive_get_icon (drive);
-	icon_name = panel_util_get_icon_name_from_g_icon (icon);
-	g_object_unref (icon);
-
 	title = g_drive_get_name (drive);
 
 	item = gtk_image_menu_item_new ();
 	setup_menu_item_with_icon (item,
 				   panel_menu_icon_get_size (),
-				   icon_name,
-				   NULL,
+				   NULL, NULL, icon,
 				   title);
+	g_object_unref (icon);
 
 	tooltip = g_strdup_printf (_("Rescan %s"), title);
 	panel_util_set_tooltip_text (item, tooltip);
 	g_free (tooltip);
 
-	g_free (icon_name);
 	g_free (title);
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -578,11 +591,12 @@ volume_mount_cb (GObject      *source_object,
 {
 	PanelVolumeMountData *mount_data = user_data;
 	GError *error;
-	char   *primary;
-	char   *name;
 
 	error = NULL;
 	if (!g_volume_mount_finish (G_VOLUME (source_object), res, &error)) {
+		char *primary;
+		char *name;
+
 		if (error->code != G_IO_ERROR_FAILED_HANDLED) {
 			name = g_volume_get_name (G_VOLUME (source_object));
 			primary = g_strdup_printf (_("Unable to mount %s"),
@@ -595,9 +609,19 @@ volume_mount_cb (GObject      *source_object,
 			g_free (primary);
 		}
 		g_error_free (error);
+	} else {
+		GMount *mount;
+		GFile  *root;
+		char   *rooturi;
+
+		mount = g_volume_get_mount (G_VOLUME (source_object));
+		root = g_mount_get_root (mount);
+		rooturi = g_file_get_uri (root);
+		activate_uri_on_screen (rooturi, mount_data->screen);
+		g_object_unref (mount);
+		g_object_unref (root);
+		g_free (rooturi);
 	}
-	
-	//FIXME: should we activate the root of the new mount?
 
 	g_object_unref (mount_data->mount_op);
 	g_slice_free (PanelVolumeMountData, mount_data);
@@ -611,9 +635,9 @@ panel_menu_item_mount_volume (GtkWidget *menuitem,
 
 	mount_data = g_slice_new (PanelVolumeMountData);
 	mount_data->screen = menuitem_to_screen (menuitem);
-	/*FIXME: kill PanelMountOperation when we can depend on newer GTK+ that
-	 * will have the same feature */
-	mount_data->mount_op = panel_mount_operation_new (NULL);
+	mount_data->mount_op = gtk_mount_operation_new (NULL);
+	gtk_mount_operation_set_screen (GTK_MOUNT_OPERATION (mount_data->mount_op),
+					mount_data->screen);
 
 	g_volume_mount (volume, G_MOUNT_MOUNT_NONE, mount_data->mount_op, NULL,
 			volume_mount_cb, mount_data);
@@ -625,28 +649,23 @@ panel_menu_item_append_volume (GtkWidget *menu,
 {
 	GtkWidget *item;
 	GIcon     *icon;
-	char      *icon_name;
 	char      *title;
 	char      *tooltip;
 
 	icon = g_volume_get_icon (volume);
-	icon_name = panel_util_get_icon_name_from_g_icon (icon);
-	g_object_unref (icon);
-
 	title = g_volume_get_name (volume);
 
 	item = gtk_image_menu_item_new ();
 	setup_menu_item_with_icon (item,
 				   panel_menu_icon_get_size (),
-				   icon_name,
-				   NULL,
+				   NULL, NULL, icon,
 				   title);
+	g_object_unref (icon);
 
 	tooltip = g_strdup_printf (_("Mount %s"), title);
 	panel_util_set_tooltip_text (item, tooltip);
 	g_free (tooltip);
 
-	g_free (icon_name);
 	g_free (title);
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -666,28 +685,24 @@ panel_menu_item_append_mount (GtkWidget *menu,
 {
 	GFile  *root;
 	GIcon  *icon;
-	char   *icon_name;
 	char   *display_name;
 	char   *activation_uri;
 
 	icon = g_mount_get_icon (mount);
-	icon_name = panel_util_get_icon_name_from_g_icon (icon);
-	g_object_unref (icon);
-
 	display_name = g_mount_get_name (mount);
 
 	root = g_mount_get_root (mount);
 	activation_uri = g_file_get_uri (root);
 	g_object_unref (root);
 
-	panel_menu_items_append_place_item (icon_name,
+	panel_menu_items_append_place_item (NULL, icon,
 					    display_name,
 					    display_name, //FIXME tooltip
 					    menu,
 					    G_CALLBACK (activate_uri),
 					    activation_uri);
 
-	g_free (icon_name);
+	g_object_unref (icon);
 	g_free (display_name);
 	g_free (activation_uri);
 }
@@ -816,6 +831,11 @@ panel_place_menu_item_append_local_gio (PanelPlaceMenuItem *place_item,
 
 		mount = l->data;
 
+		if (g_mount_is_shadowed (mount)) {
+			g_object_unref (mount);
+			continue;
+		}
+
 		volume = g_mount_get_volume (mount);
 		if (volume != NULL) {
 			g_object_unref (volume);
@@ -848,7 +868,8 @@ panel_place_menu_item_append_local_gio (PanelPlaceMenuItem *place_item,
 
 		item = gtk_image_menu_item_new ();
 		setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
-					   PANEL_ICON_REMOVABLE_MEDIA, NULL,
+					   PANEL_ICON_REMOVABLE_MEDIA,
+					   NULL, NULL,
 					   _("Removable Media"));
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -901,6 +922,12 @@ panel_place_menu_item_append_remote_gio (PanelPlaceMenuItem *place_item,
 		GFile   *root;
 
 		mount = l->data;
+
+		if (g_mount_is_shadowed (mount)) {
+			g_object_unref (mount);
+			continue;
+		}
+
 		volume = g_mount_get_volume (mount);
 		if (volume != NULL) {
 			g_object_unref (volume);
@@ -928,7 +955,8 @@ panel_place_menu_item_append_remote_gio (PanelPlaceMenuItem *place_item,
 
 		item = gtk_image_menu_item_new ();
 		setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
-					   PANEL_ICON_NETWORK_SERVER, NULL,
+					   PANEL_ICON_NETWORK_SERVER,
+					   NULL, NULL,
 					   _("Network Places"));
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -966,7 +994,7 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 	name = panel_util_get_label_for_uri (uri);
 	g_object_unref (file);
 	
-	panel_menu_items_append_place_item (PANEL_ICON_HOME,
+	panel_menu_items_append_place_item (PANEL_ICON_HOME, NULL,
 					    name,
 					    _("Open your personal folder"),
 					    places_menu,
@@ -983,12 +1011,11 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 		g_object_unref (file);
 		
 		panel_menu_items_append_place_item (
-				PANEL_ICON_DESKTOP,
+				PANEL_ICON_DESKTOP, NULL,
 				/* Translators: Desktop is used here as in
 				 * "Desktop Folder" (this is not the Desktop
-				 * environment). Do not keep "Desktop Folder|"
-				 * in the translation */
-				Q_("Desktop Folder|Desktop"),
+				 * environment). */
+				C_("Desktop Folder", "Desktop"),
 				_("Open the contents of your desktop in a folder"),
 				places_menu,
 				G_CALLBACK (activate_desktop_uri),
@@ -1009,10 +1036,6 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 	if (gconf_name)
 		g_free (gconf_name);
 
-	panel_menu_items_append_from_desktop (places_menu,
-					      "nautilus-cd-burner.desktop",
-					      NULL);
-
 	panel_place_menu_item_append_local_gio (place_item, places_menu);
 	add_menu_separator (places_menu);
 
@@ -1483,11 +1506,15 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 		}
 	}
 
-	/* Translators: translate "panel:showusername|1" to anything
+	if (panel_lockdown_get_disable_log_out ())
+		return;
+	/* Below this, we only have log out/shutdown items */
+
+	/* Translators: translate "1" (msgctxt: "panel:showusername") to anything
 	 * but "1" if "Log Out %s" doesn't make any sense in your
 	 * language (where %s is a username).
 	 */
-	translate = Q_("panel:showusername|1");
+	translate = C_("panel:showusername", "1");
 	if (strcmp (translate, "1") == 0) {
 		const char *user_name;
 
@@ -1498,11 +1525,11 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 		/* keep those strings in sync with the ones in
 		 * panel-action-button.c */
 		/* Translators: this string is used ONLY if you translated
-		 * "panel:showusername|1" to "1" */
+		 * "1" (msgctxt: "panel:showusername") to "1" */
 		label = g_strdup_printf (_("Log Out %s..."),
 					 g_get_user_name ());
 		/* Translators: this string is used ONLY if you translated
-		 * "panel:showusername|1" to "1" */
+		 * "1" (msgctxt: "panel:showusername") to "1" */
 		tooltip = g_strdup_printf (_("Log out %s of this session to "
 					     "log in as a different user"),
 					   user_name);
@@ -1517,8 +1544,10 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 	g_free (tooltip);
 
 	if (item != NULL) {
-		if (!separator_inserted)
+		if (!separator_inserted) {
 			add_menu_separator (menu);
+			separator_inserted = TRUE;
+		}
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
@@ -1536,17 +1565,5 @@ void
 panel_menu_item_activate_desktop_file (GtkWidget  *menuitem,
 				       const char *path)
 {
-	GError *error;
-
-	error = NULL;
-	panel_launch_desktop_file (path, NULL,
-				   menuitem_to_screen (menuitem), &error);
-	if (error) {
-		panel_error_dialog (NULL, menuitem_to_screen (menuitem),
-				    "cannot_launch_entry", TRUE,
-				    _("Could not launch menu item"),
-				    error->message);
-
-		g_error_free (error);
-	}
+	panel_launch_desktop_file (path, menuitem_to_screen (menuitem), NULL);
 }
