@@ -17,6 +17,8 @@
 #include "panel-util.h"
 
 #include <dirent.h>
+#include <errno.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -27,10 +29,6 @@
 #include <gio/gio.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
-#include <libgnome/gnome-desktop-item.h>
-#include <libgnome/gnome-util.h>
-#include <libgnomeui/gnome-help.h>
-
 #include <libpanel-util/panel-error.h>
 #include <libpanel-util/panel-glib.h>
 #include <libpanel-util/panel-keyfile.h>
@@ -45,110 +43,6 @@
 #include "panel-icon-names.h"
 #include "panel-lockdown.h"
 
-static int
-panel_ditem_launch (GnomeDesktopItem  *item,
-		    GList             *file_list,
-		    GdkScreen         *screen,
-		    GError           **error)
-{
-	int workspace;
-
-	workspace = xstuff_get_current_workspace (screen);
-
-	gnome_desktop_item_set_launch_time (item,
-					    gtk_get_current_event_time ());
-
-	return gnome_desktop_item_launch_on_screen (item, file_list, 0,
-						    screen, workspace, error);
-}
-
-void
-panel_util_launch_from_key_file (GKeyFile   *keyfile,
-				 GList      *file_list,
-				 GdkScreen  *screen,
-				 GError    **error)
-{
-	GnomeDesktopItem  *ditem;
-	int                i;
-	static const char *keys [] = { "Type",
-				       "Exec",
-				       "URL",
-				       "Dev",
-				       "Path",
-				       "Terminal",
-				       "TerminalOptions",
-				       "StartupNotify",
-				       "StartupWMClass",
-				     };
-	static const char *locale_keys [] = { "Name",
-					      "GenericName",
-					      "Icon",
-					      "MiniIcon",
-					    };
-
-	g_return_if_fail (keyfile != NULL);
-
-	ditem = gnome_desktop_item_new ();
-	if (ditem == NULL)
-		return;
-
-	for (i = 0; i < G_N_ELEMENTS (keys); i++) {
-		char *value;
-
-		value = panel_key_file_get_string (keyfile, keys [i]);
-		if (value != NULL) {
-			gnome_desktop_item_set_string (ditem, keys [i], value);
-			g_free (value);
-		}
-	}
-
-	for (i = 0; i < G_N_ELEMENTS (locale_keys); i++) {
-		char *value;
-
-		value = panel_key_file_get_locale_string (keyfile,
-							  locale_keys [i]);
-		if (value != NULL) {
-			gnome_desktop_item_set_string (ditem, locale_keys [i], value);
-			g_free (value);
-		}
-	}
-
-	panel_ditem_launch (ditem, file_list, screen, error);
-	gnome_desktop_item_unref (ditem);
-}
-
-void
-panel_launch_desktop_file (const char  *desktop_file,
-			   const char  *fallback_exec,
-			   GdkScreen   *screen,
-			   GError     **error)
-{
-	GnomeDesktopItem *ditem;
-
-	if (g_path_is_absolute (desktop_file))
-		ditem = gnome_desktop_item_new_from_file (desktop_file, 0,
-							  error);
-	else
-		ditem = gnome_desktop_item_new_from_basename (desktop_file, 0,
-							      error);
-
-	if (ditem != NULL) {
-		panel_ditem_launch (ditem, NULL, screen, error);
-		gnome_desktop_item_unref (ditem);
-	} else if (fallback_exec != NULL) {
-		char *argv [2] = {(char *)fallback_exec, NULL};
-
-		if (*error) {
-			g_error_free (*error);
-			*error = NULL;
-		}
-
-		gdk_spawn_on_screen (screen, NULL, argv, NULL,
-				     G_SPAWN_SEARCH_PATH,
-				     NULL, NULL, NULL, error);
-	}
-}
-
 char *
 panel_util_make_exec_uri_for_desktop (const char *exec)
 {
@@ -180,185 +74,6 @@ panel_util_make_exec_uri_for_desktop (const char *exec)
 	return g_string_free (str, FALSE);
 }
 
-void
-panel_show_help (GdkScreen  *screen,
-		 const char *doc_name,
-		 const char *linkid)
-{
-	GError *error = NULL;
-
-	if (!gnome_help_display_desktop_on_screen (NULL, "user-guide", doc_name, linkid, screen, &error)) {
-		panel_error_dialog (NULL, screen, "cannot_show_help", TRUE,
-				    _("Could not display help document"),
-				    error != NULL ? error->message : NULL);
-
-		g_clear_error (&error);
-	}
-}
-
-GList *
-panel_g_list_insert_before (GList *list,
-			    GList *sibling,
-			    GList *link)
-{
-	if (!list) {
-		g_return_val_if_fail (sibling == NULL, list);
-		return link;
-	} else if (sibling) {
-		if (sibling->prev) {
-			link->prev = sibling->prev;
-			link->prev->next = link;
-			link->next = sibling;
-			sibling->prev = link;
-			return list;
-		} else {
-			link->next = sibling;
-			sibling->prev = link;
-			g_return_val_if_fail (sibling == list, link);
-			return link;
-		}
-	} else {
-		GList *last;
-
-		last = list;
-		while (last->next)
-			last = last->next;
-
-		last->next = link;
-		link->prev = last;
-		return list;
-	}
-}
-
-GList *
-panel_g_list_insert_after (GList *list,
-			   GList *sibling,
-			   GList *link)
-{
-	if (!list) {
-		g_return_val_if_fail (sibling == NULL, link);
-		return link;
-	} else if (sibling) {
-		if (sibling->next) {
-			link->next = sibling->next;
-			link->next->prev = link;
-			link->prev = sibling;
-			sibling->next = link;
-			return list;
-		} else {
-			sibling->next = link;
-			link->prev = sibling;
-			return list;
-		}
-			
-	} else {
-		link->next = list;
-		list->prev = link;
-		return link;
-	}
-}
-
-GList *
-panel_g_list_swap_next (GList *list, GList *dl)
-{
-	GList *t;
-
-	if(!dl->next)
-		return list;
-	if(dl->prev)
-		dl->prev->next = dl->next;
-	t = dl->prev;
-	dl->prev = dl->next;
-	dl->next->prev = t;
-	if(dl->next->next)
-		dl->next->next->prev = dl;
-	t = dl->next->next;
-	dl->next->next = dl;
-	dl->next = t;
-
-	if(list == dl)
-		return dl->prev;
-	return list;
-}
-
-GList *
-panel_g_list_swap_prev (GList *list, GList *dl)
-{
-	GList *t;
-
-	if(!dl->prev)
-		return list;
-	if(dl->next)
-		dl->next->prev = dl->prev;
-	t = dl->next;
-	dl->next = dl->prev;
-	dl->prev->next = t;
-	if(dl->prev->prev)
-		dl->prev->prev->next = dl;
-	t = dl->prev->prev;
-	dl->prev->prev = dl;
-	dl->prev = t;
-
-	if(list == dl->next)
-		return dl;
-	return list;
-}
-
-/*maybe this should be a glib function?
- it resorts a single item in the list*/
-GList *
-panel_g_list_resort_item(GList *list, gpointer data, GCompareFunc func)
-{
-	GList *dl;
-
-	if(!list)
-		return NULL;
-
-	dl = g_list_find(list,data);
-
-	g_return_val_if_fail(dl!=NULL,list);
-
-	while(dl->next &&
-	      (*func)(dl->data,dl->next->data)>0)
-		list = panel_g_list_swap_next (list, dl);
-	while(dl->prev &&
-	      (*func)(dl->data,dl->prev->data)<0)
-		list = panel_g_list_swap_prev (list, dl);
-	return list;
-}
-
-GSList *
-panel_g_slist_make_unique (GSList       *list,
-			   GCompareFunc  compare,
-			   gboolean      free_data)
-{
-	GSList *sorted, *l;
-
-	g_assert (compare != NULL);
-
-	if (!list)
-		return NULL;
-
-	sorted = g_slist_copy (list);
-	sorted = g_slist_sort (sorted, compare);
-
-	for (l = sorted; l; l = l->next) {
-		GSList *next;
-
-		next = l->next;
-		if (l->data && next && next->data)
-			if (!compare (l->data, next->data)) {
-				list = g_slist_remove (list, l->data);
-				if (free_data)
-					g_free (l->data);
-			}
-	}
-
-	g_slist_free (sorted);
-
-	return list;
-}
-
 int
 panel_find_applet_index (GtkWidget *widget)
 {
@@ -727,6 +442,106 @@ panel_lock_screen (GdkScreen *screen)
 }
 
 
+#define PANEL_LAUNCHER_PERSONAL_PATH "panel2.d/default/launchers"
+
+static char *
+panel_launcher_get_personal_path (void)
+{
+	return g_build_filename (g_get_home_dir (), ".gnome2",
+				 PANEL_LAUNCHER_PERSONAL_PATH, NULL);
+}
+
+gboolean
+panel_launcher_is_in_personal_path (const char *location)
+{
+	GFile    *file;
+	GFile    *launchers_dir;
+	char     *launchers_path;
+	gboolean  retval;
+
+	if (!location)
+		return FALSE;
+
+	launchers_path = panel_launcher_get_personal_path ();
+	launchers_dir = g_file_new_for_path (launchers_path);
+	g_free (launchers_path);
+
+	file = panel_launcher_get_gfile (location);
+
+	retval = g_file_has_prefix (file, launchers_dir);
+
+	g_object_unref (file);
+	g_object_unref (launchers_dir);
+
+	return retval;
+}
+
+GFile *
+panel_launcher_get_gfile (const char *location)
+{
+	char  *path;
+	GFile *file;
+
+	if (!g_ascii_strncasecmp (location, "file:", strlen ("file:")))
+		return g_file_new_for_uri (location);
+
+	if (g_path_is_absolute (location))
+		return g_file_new_for_path (location);
+
+	path = panel_make_full_path (NULL, location);
+	file = g_file_new_for_path (path);
+	g_free (path);
+
+	return file;
+}
+
+char *
+panel_launcher_get_uri (const char *location)
+{
+	char *path;
+	char *uri;
+
+	if (!g_ascii_strncasecmp (location, "file:", strlen ("file:")))
+		return g_strdup (location);
+
+	if (!g_path_is_absolute (location))
+		path = panel_make_full_path (NULL, location);
+	else
+		path = g_strdup (location);
+
+	uri = g_filename_to_uri (path, NULL, NULL);
+	g_free (path);
+
+	return uri;
+}
+
+char *
+panel_launcher_get_filename (const char *location)
+{
+	GFile *file;
+	GFile *launchers_dir;
+	char  *launchers_path;
+	char  *retval;
+
+	if (!g_path_is_absolute (location) &&
+	    g_ascii_strncasecmp (location, "file:", strlen ("file:")))
+		/* this is not a local URI */
+		return NULL;
+
+	launchers_path = panel_launcher_get_personal_path ();
+	launchers_dir = g_file_new_for_path (launchers_path);
+	g_free (launchers_path);
+
+	file = panel_launcher_get_gfile (location);
+
+	retval = g_file_get_relative_path (launchers_dir, file);
+
+	g_object_unref (file);
+	g_object_unref (launchers_dir);
+
+	return retval;
+}
+
 char *
 panel_make_full_path (const char *dir,
 		      const char *filename)
@@ -737,7 +552,7 @@ panel_make_full_path (const char *dir,
 	g_return_val_if_fail (filename != NULL, NULL);
 
 	if (!dir) {
-		freeme = gnome_util_home_file (PANEL_LAUNCHERS_PATH);
+		freeme = panel_launcher_get_personal_path ();
 		dir = freeme;
 	}
 
