@@ -6,53 +6,10 @@
 
 G_BEGIN_DECLS
 
-typedef void (*UpdateFunction) (gpointer);
-
-/* TRUE if string is NULL or the first character is '\0' */
-#define		string_empty(s)		((s)==NULL||((char *)(s))[0]=='\0')
-
 #define		sure_string(s)		((const char *)((s)!=NULL?(s):""))
 
-void            panel_util_launch_from_key_file (GKeyFile                *keyfile,
-						 GList                   *file_list,
-						 GdkScreen               *screen,
-						 GError                 **error);
-void            panel_launch_desktop_file  (const char                   *desktop_file,
-					    const char                   *fallback_exec,
-					    GdkScreen                    *screen,
-					    GError                      **error);
-
 char *          panel_util_make_exec_uri_for_desktop (const char *exec);
 
-void		panel_show_help		(GdkScreen  *screen,
-					 const char *path,
-					 const char *linkid);
-
-GList *panel_g_list_insert_before (GList        *list,
-				   GList        *sibling,
-				   GList        *link);
-GList *panel_g_list_insert_after  (GList        *list,
-				   GList        *sibling,
-				   GList        *link);
-GList *panel_g_list_swap_next     (GList        *list,
-				   GList        *dl);
-GList *panel_g_list_swap_prev     (GList        *list,
-				   GList        *dl);
-GList *panel_g_list_resort_item   (GList        *list,
-				   gpointer      data,
-				   GCompareFunc  func);
-
-GSList *panel_g_slist_make_unique (GSList       *list,
-				   GCompareFunc  compare,
-				   gboolean      free_data);
-
-GtkWidget      *panel_error_dialog      (GtkWindow  *parent,
-					 GdkScreen  *screen,
-					 const char *class,
-					 gboolean    auto_destroy,
-					 const char *primary_text,
-					 const char *secondary_text);
-
 int		panel_find_applet_index	(GtkWidget *widget);
 
 void		panel_push_window_busy	(GtkWidget *window);
@@ -79,16 +36,17 @@ GdkPixbuf *     panel_load_icon         (GtkIconTheme  *icon_theme,
 					 int            desired_height,
 					 char         **error_msg);
 
+GFile      *panel_launcher_get_gfile           (const char *location);
+char       *panel_launcher_get_uri             (const char *location);
+char       *panel_launcher_get_filename        (const char *location);
+gboolean    panel_launcher_is_in_personal_path (const char *location);
+
 char *panel_make_full_path   (const char *dir,
 			      const char *filename);
 char *panel_make_unique_desktop_path_from_name (const char *dir,
 						const char *name);
 char *panel_make_unique_desktop_uri (const char *dir,
 				     const char *source);
-char *panel_lookup_in_data_dirs (const char *basename);
-
-const char *panel_util_utf8_strstrcase (const char *haystack,
-					const char *needle);
 
 GdkPixbuf *panel_util_cairo_rgbdata_to_pixbuf (unsigned char *data,
 					       int            width,
