@@ -29,11 +29,12 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+
+#include <cairo.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkbindings.h>
-#include <gtk/gtktooltips.h>
+#include <gtk/gtk.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-types.h>
@@ -41,8 +42,6 @@
 #include <bonobo/bonobo-item-handler.h>
 #include <bonobo/bonobo-shlib-factory.h>
 #include <bonobo/bonobo-property-bag-client.h>
-#include <libgnome/gnome-program.h>
-#include <libgnomeui/gnome-ui-init.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 #include <X11/Xatom.h>
@@ -71,6 +70,10 @@ struct _PanelAppletPrivate {
 	PanelAppletOrient           orient;
 	guint                       size;
 	char                       *background;
+	GtkWidget                  *background_widget;
+
+	int                         previous_width;
+	int                         previous_height;
 
         int                        *size_hints;
         int                         size_hints_len;
@@ -80,8 +83,6 @@ struct _PanelAppletPrivate {
 	gboolean                    locked_down;
 };
 
-static GObjectClass *parent_class;
-
 enum {
         CHANGE_ORIENT,
         CHANGE_SIZE,
@@ -109,6 +110,11 @@ enum {
 	PROPERTY_LOCKED_DOWN_IDX
 };
 
+G_DEFINE_TYPE (PanelApplet, panel_applet, GTK_TYPE_EVENT_BOX)
+
+static void panel_applet_handle_background (PanelApplet *applet);
+static void panel_applet_setup             (PanelApplet *applet);
+
 static void
 panel_applet_associate_schemas_in_dir (GConfClient  *client,
 				       const gchar  *prefs_key,
@@ -141,7 +147,7 @@ panel_applet_associate_schemas_in_dir (GConfClient  *client,
 
 		g_free (key);
 
-		gconf_entry_free (entry);
+		gconf_entry_unref (entry);
 
 		if (*error) {
 			g_slist_free (list);
@@ -552,7 +558,7 @@ panel_applet_finalize (GObject *object)
 		g_closure_unref (applet->priv->closure);
 	applet->priv->closure = NULL;
 
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (panel_applet_parent_class)->finalize (object);
 }
 
 static gboolean
@@ -611,8 +617,19 @@ panel_applet_position_menu (GtkMenu   *menu,
 
 	if (applet->priv->orient == PANEL_APPLET_ORIENT_UP ||
 	    applet->priv->orient == PANEL_APPLET_ORIENT_DOWN) {
-		if (requisition.width < pointer_x)
-			menu_x += MIN (pointer_x, widget->allocation.width - requisition.width);
+		if (gtk_widget_get_direction (GTK_WIDGET (menu)) != GTK_TEXT_DIR_RTL) {
+			if (pointer_x < widget->allocation.width &&
+			    requisition.width < pointer_x)
+				menu_x += MIN (pointer_x,
+					       widget->allocation.width - requisition.width);
+		} else {
+			menu_x += widget->allocation.width - requisition.width;
+			if (pointer_x > 0 && pointer_x < widget->allocation.width &&
+			    pointer_x < widget->allocation.width - requisition.width) {
+				menu_x -= MIN (widget->allocation.width - pointer_x,
+					       widget->allocation.width - requisition.width);
+			}
+		}
 		menu_x = MIN (menu_x, gdk_screen_get_width (screen) - requisition.width);
 
 		if (menu_y > gdk_screen_get_height (screen) / 2)
@@ -620,7 +637,8 @@ panel_applet_position_menu (GtkMenu   *menu,
 		else
 			menu_y += widget->allocation.height;
 	} else  {
-		if (requisition.height < pointer_y)
+		if (pointer_y < widget->allocation.height &&
+		    requisition.height < pointer_y)
 			menu_y += MIN (pointer_y, widget->allocation.height - requisition.height);
 		menu_y = MIN (menu_y, gdk_screen_get_height (screen) - requisition.height);
 
@@ -643,7 +661,7 @@ panel_applet_can_focus (GtkWidget *widget)
 	 * A PanelApplet widget can focus if it has a tooltip or it does 
 	 * not have any focusable children.
 	 */
-	if (gtk_tooltips_data_get (widget))
+	if (gtk_widget_get_has_tooltip (widget))
 		return TRUE;
 
 	if (!PANEL_IS_APPLET (widget))
@@ -703,7 +721,8 @@ panel_applet_size_request (GtkWidget *widget, GtkRequisition *requisition)
 {
 	int focus_width = 0;
 
-	GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
+	GTK_WIDGET_CLASS (panel_applet_parent_class)->size_request (widget,
+								    requisition);
 
 	if (!panel_applet_can_focus (widget))
 		return;
@@ -728,43 +747,53 @@ panel_applet_size_allocate (GtkWidget     *widget,
 	GtkBin        *bin;
 	int            border_width;
 	int            focus_width = 0;
+	PanelApplet   *applet;
 
 	if (!panel_applet_can_focus (widget)) {
-		GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
-		return;
+		GTK_WIDGET_CLASS (panel_applet_parent_class)->size_allocate (widget, allocation);
+	} else {
+		/*
+		 * We are deliberately ignoring focus-padding here to
+		 * save valuable panel real estate.
+		 */
+		gtk_widget_style_get (widget,
+				      "focus-line-width", &focus_width,
+				      NULL);
+
+		border_width = GTK_CONTAINER (widget)->border_width;
+
+		widget->allocation = *allocation;
+		bin = GTK_BIN (widget);
+
+		child_allocation.x = focus_width;
+		child_allocation.y = focus_width;
+
+		child_allocation.width  = MAX (allocation->width  - border_width * 2, 0);
+		child_allocation.height = MAX (allocation->height - border_width * 2, 0);
+
+		if (GTK_WIDGET_REALIZED (widget))
+			gdk_window_move_resize (widget->window,
+						allocation->x + GTK_CONTAINER (widget)->border_width,
+						allocation->y + GTK_CONTAINER (widget)->border_width,
+						child_allocation.width,
+						child_allocation.height);
+
+		child_allocation.width  = MAX (child_allocation.width  - 2 * focus_width, 0);
+		child_allocation.height = MAX (child_allocation.height - 2 * focus_width, 0);
+
+		if (bin->child)
+			gtk_widget_size_allocate (bin->child, &child_allocation);
 	}
 
-	/*
-	 * We are deliberately ignoring focus-padding here to
-	 * save valuable panel real estate.
-	 */
-	gtk_widget_style_get (widget,
-			      "focus-line-width", &focus_width,
-			      NULL);
+	applet = PANEL_APPLET (widget);
 
-	border_width = GTK_CONTAINER (widget)->border_width;
+	if (applet->priv->previous_height != allocation->height ||
+	    applet->priv->previous_width  != allocation->width) {
+		applet->priv->previous_height = allocation->height;
+		applet->priv->previous_width = allocation->width;
 
-	widget->allocation = *allocation;
-	bin = GTK_BIN (widget);
-
- 	child_allocation.x = focus_width;
- 	child_allocation.y = focus_width;
-
-	child_allocation.width  = MAX (allocation->width  - border_width * 2, 0);
-	child_allocation.height = MAX (allocation->height - border_width * 2, 0);
-
-	if (GTK_WIDGET_REALIZED (widget))
-		gdk_window_move_resize (widget->window,
-					allocation->x + GTK_CONTAINER (widget)->border_width,
-					allocation->y + GTK_CONTAINER (widget)->border_width,
-					child_allocation.width,
-					child_allocation.height);
-
-	child_allocation.width  = MAX (child_allocation.width  - 2 * focus_width, 0);
-	child_allocation.height = MAX (child_allocation.height - 2 * focus_width, 0);
-
-	if (bin->child)
-		gtk_widget_size_allocate (bin->child, &child_allocation);
+		panel_applet_handle_background (applet);
+	}
 }
 
 static gboolean
@@ -778,7 +807,8 @@ panel_applet_expose (GtkWidget      *widget,
 	g_return_val_if_fail (PANEL_IS_APPLET (widget), FALSE);
 	g_return_val_if_fail (event != NULL, FALSE);
 
-	GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+	GTK_WIDGET_CLASS (panel_applet_parent_class)->expose_event (widget,
+								    event);
 
         if (!GTK_WIDGET_HAS_FOCUS (widget))
 		return FALSE;
@@ -828,23 +858,21 @@ panel_applet_focus (GtkWidget        *widget,
 	}
 
 	previous_focus_child = GTK_CONTAINER (widget)->focus_child;
-	 if (!previous_focus_child && !GTK_WIDGET_HAS_FOCUS (widget)) {
-		GtkTooltipsData *tooltip;
-
-		tooltip = gtk_tooltips_data_get (widget);
-		if (tooltip) {
+	if (!previous_focus_child && !GTK_WIDGET_HAS_FOCUS (widget)) {
+		if (gtk_widget_get_has_tooltip (widget)) {
 			GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
 			gtk_widget_grab_focus (widget);
 			GTK_WIDGET_UNSET_FLAGS (widget, GTK_CAN_FOCUS);
 			return TRUE;
 		}
 	}
-	ret = GTK_WIDGET_CLASS (parent_class)->focus (widget, dir);
+	ret = GTK_WIDGET_CLASS (panel_applet_parent_class)->focus (widget, dir);
+
 	if (!ret && !previous_focus_child) {
  		if (!GTK_WIDGET_HAS_FOCUS (widget))  {
 			/*
 			 * Applet does not have a widget which can focus so set
-			 * the focus on the applet unless it aleready had focus
+			 * the focus on the applet unless it already had focus
 			 * because it had a tooltip.
 			 */ 
 			GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
@@ -925,47 +953,64 @@ panel_applet_get_pixmap (PanelApplet     *applet,
 			 int              x,
 			 int              y)
 {
-	GdkPixmap  *pixmap;
-	GdkDisplay *display;
-	GdkPixmap  *retval;
-	GdkGC      *gc;
-	int         width;
-	int         height;
+	gboolean         display_grabbed;
+	GdkPixmap       *pixmap;
+	GdkDisplay      *display;
+	GdkPixmap       *retval;
+	int              width;
+	int              height;
+	cairo_t         *cr;
+	cairo_pattern_t *pattern;
 
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
 
 	if (!GTK_WIDGET_REALIZED (applet))
 		return NULL;
 
-	gc = gdk_gc_new (GDK_DRAWABLE (GTK_WIDGET (applet)->window));
-
-	g_return_val_if_fail (GDK_IS_GC (gc), NULL);
-
 	display = gdk_display_get_default ();
-	
+	display_grabbed = FALSE;
+
 	pixmap = gdk_pixmap_lookup_for_display (display, xid);
 	if (pixmap)
 		g_object_ref (pixmap);
-	else
+	else {
+		display_grabbed = TRUE;
+		gdk_x11_display_grab (display);
 		pixmap = gdk_pixmap_foreign_new_for_display (display, xid);
+	}
 
-	g_return_val_if_fail (pixmap != NULL, NULL);
+	/* This can happen if the user changes the background very fast.
+	 * We'll get the next update, so it's not a big deal. */
+	if (pixmap == NULL) {
+		if (display_grabbed)
+			gdk_x11_display_ungrab (display);
+		return NULL;
+	}
 
 	gdk_drawable_get_size (GDK_DRAWABLE (GTK_WIDGET (applet)->window),
 			       &width, &height);
+	retval = gdk_pixmap_new (GTK_WIDGET (applet)->window,
+				 width, height, -1);
 
-	retval = gdk_pixmap_new (GTK_WIDGET (applet)->window, width, height, -1);
+	/* the pixmap has no colormap, and we need one */
+	gdk_drawable_set_colormap (GDK_DRAWABLE (pixmap),
+				   gdk_drawable_get_colormap (GTK_WIDGET (applet)->window));
 
-	gdk_draw_drawable (GDK_DRAWABLE (retval),
-			   gc, 
-			   GDK_DRAWABLE (pixmap),
-			   x, y,
-			   0, 0,
-			   width, height);
+	cr = gdk_cairo_create (GDK_DRAWABLE (retval));
+	gdk_cairo_set_source_pixmap (cr, pixmap, -x, -y);
+	pattern = cairo_get_source (cr);
+	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+
+	cairo_rectangle (cr, 0, 0, width, height);
+	cairo_fill (cr);
+
+	cairo_destroy (cr);
 
-	g_object_unref (gc);
 	g_object_unref (pixmap);
 
+	if (display_grabbed)
+		gdk_x11_display_ungrab (display);
+
 	return retval;
 }
 
@@ -992,7 +1037,7 @@ panel_applet_handle_background_string (PanelApplet  *applet,
 
 		if (!elements [1] || !panel_applet_parse_color (elements [1], color)) {
 
-			g_warning (_("Incomplete '%s' background type received"), elements [0]);
+			g_warning ("Incomplete '%s' background type received", elements [0]);
 			g_strfreev (elements);
 			return PANEL_NO_BACKGROUND;
 		}
@@ -1006,7 +1051,7 @@ panel_applet_handle_background_string (PanelApplet  *applet,
 		g_return_val_if_fail (pixmap != NULL, PANEL_NO_BACKGROUND);
 
 		if (!panel_applet_parse_pixmap_str (elements [1], &pixmap_id, &x, &y)) {
-			g_warning (_("Incomplete '%s' background type received: %s"),
+			g_warning ("Incomplete '%s' background type received: %s",
 				   elements [0], elements [1]);
 
 			g_strfreev (elements);
@@ -1015,14 +1060,14 @@ panel_applet_handle_background_string (PanelApplet  *applet,
 
 		*pixmap = panel_applet_get_pixmap (applet, pixmap_id, x, y);
 		if (!*pixmap) {
-			g_warning (_("Failed to get pixmap %s"), elements [1]);
+			g_warning ("Failed to get pixmap %s", elements [1]);
 			g_strfreev (elements);
 			return PANEL_NO_BACKGROUND;
 		}
 
 		retval = PANEL_PIXMAP_BACKGROUND;
 	} else
-		g_warning (_("Unknown background type received"));
+		g_warning ("Unknown background type received");
 
 	g_strfreev (elements);
 
@@ -1090,14 +1135,53 @@ panel_applet_get_prop (BonoboPropertyBag *sack,
 	}
 }
 
+static void
+panel_applet_update_background_for_widget (GtkWidget                 *widget,
+					   PanelAppletBackgroundType  type,
+					   GdkColor                  *color,
+					   GdkPixmap                 *pixmap)
+{
+	GtkRcStyle *rc_style;
+	GtkStyle   *style;
+
+	/* reset style */
+	gtk_widget_set_style (widget, NULL);
+	rc_style = gtk_rc_style_new ();
+	gtk_widget_modify_style (widget, rc_style);
+	g_object_unref (rc_style);
+
+	switch (type) {
+	case PANEL_NO_BACKGROUND:
+		break;
+	case PANEL_COLOR_BACKGROUND:
+		gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, color);
+		break;
+	case PANEL_PIXMAP_BACKGROUND:
+		style = gtk_style_copy (widget->style);
+		if (style->bg_pixmap[GTK_STATE_NORMAL])
+			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
+		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
+		gtk_widget_set_style (widget, style);
+		g_object_unref (style);
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+}
+
 static void
 panel_applet_handle_background (PanelApplet *applet)
 {
 	PanelAppletBackgroundType  type;
 	GdkColor                   color;
-	GdkPixmap                 *pixmap = NULL;
+	GdkPixmap                 *pixmap;
 
-	type = panel_applet_handle_background_string (applet, &color, &pixmap);
+	type = panel_applet_get_background (applet, &color, &pixmap);
+
+	if (applet->priv->background_widget)
+		panel_applet_update_background_for_widget (applet->priv->background_widget,
+							   type, &color, pixmap);
 
 	switch (type) {
 	case PANEL_NO_BACKGROUND:
@@ -1207,7 +1291,7 @@ panel_applet_property_bag (PanelApplet *applet)
 				 PROPERTY_ORIENT_IDX,
 				 BONOBO_ARG_SHORT,
 				 NULL,
-				 _("The Applet's containing Panel's orientation"),
+				 "The Applet's containing Panel's orientation",
 				 Bonobo_PROPERTY_READABLE | Bonobo_PROPERTY_WRITEABLE);
 
 	bonobo_property_bag_add (sack,
@@ -1215,7 +1299,7 @@ panel_applet_property_bag (PanelApplet *applet)
 				 PROPERTY_SIZE_IDX,
 				 BONOBO_ARG_SHORT,
 				 NULL,
-				 _("The Applet's containing Panel's size in pixels"),
+				 "The Applet's containing Panel's size in pixels",
 				 Bonobo_PROPERTY_READABLE | Bonobo_PROPERTY_WRITEABLE);
 
 	bonobo_property_bag_add (sack,
@@ -1223,7 +1307,7 @@ panel_applet_property_bag (PanelApplet *applet)
 				 PROPERTY_BACKGROUND_IDX,
 				 BONOBO_ARG_STRING,
 				 NULL,
-				 _("The Applet's containing Panel's background color or pixmap"),
+				 "The Applet's containing Panel's background color or pixmap",
 				 Bonobo_PROPERTY_READABLE | Bonobo_PROPERTY_WRITEABLE);
 
 	bonobo_property_bag_add (sack,
@@ -1231,7 +1315,7 @@ panel_applet_property_bag (PanelApplet *applet)
 				 PROPERTY_FLAGS_IDX,
 				 BONOBO_ARG_SHORT,
 				 NULL,
-				 _("The Applet's flags"),
+				 "The Applet's flags",
 				 Bonobo_PROPERTY_READABLE);
 
 	bonobo_property_bag_add (sack,
@@ -1239,7 +1323,7 @@ panel_applet_property_bag (PanelApplet *applet)
 				 PROPERTY_SIZE_HINTS_IDX,
 				 TC_CORBA_sequence_CORBA_long,
 				 NULL,
-				 _("Ranges that hint what sizes are acceptable for the applet"),
+				 "Ranges that hint what sizes are acceptable for the applet",
 				 Bonobo_PROPERTY_READABLE);
 
 	bonobo_property_bag_add (sack,
@@ -1247,7 +1331,7 @@ panel_applet_property_bag (PanelApplet *applet)
 				 PROPERTY_LOCKED_DOWN_IDX,
 				 BONOBO_ARG_BOOLEAN,
 				 NULL,
-				 _("The Applet's containing Panel is locked down"),
+				 "The Applet's containing Panel is locked down",
 				 Bonobo_PROPERTY_READABLE | Bonobo_PROPERTY_WRITEABLE);
 
 	return sack;
@@ -1256,7 +1340,7 @@ panel_applet_property_bag (PanelApplet *applet)
 static void
 panel_applet_realize (GtkWidget *widget)
 {
-	GTK_WIDGET_CLASS (parent_class)->realize (widget);
+	GTK_WIDGET_CLASS (panel_applet_parent_class)->realize (widget);
 
 	if (PANEL_APPLET (widget)->priv->background)
 		panel_applet_handle_background (PANEL_APPLET (widget));
@@ -1420,6 +1504,25 @@ add_tab_bindings (GtkBindingSet   *binding_set,
 				      GTK_TYPE_DIRECTION_TYPE, direction);
 }
 
+static GObject *
+panel_applet_constructor (GType                  type,
+			  guint                  n_construct_properties,
+			  GObjectConstructParam *construct_properties)
+{
+	GObject     *obj;
+	PanelApplet *applet;
+
+	obj = G_OBJECT_CLASS (panel_applet_parent_class)->constructor (type,
+								       n_construct_properties,
+								       construct_properties);
+
+	applet = PANEL_APPLET (obj);
+
+	panel_applet_setup (applet);
+
+	return obj;
+}
+
 static void
 panel_applet_class_init (PanelAppletClass *klass)
 {
@@ -1428,8 +1531,7 @@ panel_applet_class_init (PanelAppletClass *klass)
 	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
 	GtkBindingSet *binding_set;
 
-	parent_class = g_type_class_peek_parent (klass);
-
+	gobject_class->constructor = panel_applet_constructor;
 	klass->move_focus_out_of_applet = panel_applet_move_focus_out_of_applet;
 
 	widget_class->button_press_event = panel_applet_button_press;
@@ -1501,8 +1603,7 @@ panel_applet_class_init (PanelAppletClass *klass)
 }
 
 static void
-panel_applet_instance_init (PanelApplet      *applet,
-			    PanelAppletClass *klass)
+panel_applet_init (PanelApplet *applet)
 {
 	applet->priv = PANEL_APPLET_GET_PRIVATE (applet);
 
@@ -1519,32 +1620,6 @@ panel_applet_instance_init (PanelApplet      *applet,
 			       GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
 }
 
-GType
-panel_applet_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (PanelAppletClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) panel_applet_class_init,
-			NULL,
-			NULL,
-			sizeof (PanelApplet),
-			0,
-			(GInstanceInitFunc) panel_applet_instance_init,
-			NULL
-		};
-
-		type = g_type_register_static (GTK_TYPE_EVENT_BOX, "PanelApplet",
-					       &info, 0);
-	}
-
-	return type;
-}
-
 static void
 panel_applet_setup (PanelApplet *applet)
 {
@@ -1586,8 +1661,6 @@ panel_applet_new (void)
 
 	applet = g_object_new (PANEL_TYPE_APPLET, NULL);
 
-	panel_applet_setup (applet);
-
 	return GTK_WIDGET (applet);
 }
 
@@ -1626,8 +1699,6 @@ panel_applet_factory_callback (BonoboGenericFactory    *factory,
 
 	applet = g_object_new (data->applet_type, NULL);
 
-	panel_applet_setup (applet);
-
 	applet->priv->iid     = g_strdup (iid);
 	applet->priv->closure = g_closure_ref (data->closure);
 
@@ -1742,3 +1813,22 @@ panel_applet_shlib_factory (const char                 *iid,
 					user_data, NULL),
 			ev);
 }
+
+void
+panel_applet_set_background_widget (PanelApplet *applet,
+				    GtkWidget   *widget)
+{
+	applet->priv->background_widget = widget;
+
+	if (widget) {
+		PanelAppletBackgroundType  type;
+		GdkColor                   color;
+		GdkPixmap                 *pixmap;
+
+		type = panel_applet_get_background (applet, &color, &pixmap);
+		panel_applet_update_background_for_widget (widget, type,
+							   &color, pixmap);
+		if (type == PANEL_PIXMAP_BACKGROUND)
+			g_object_unref (pixmap);
+	}
+}
